@@ -387,6 +387,7 @@
             scr-restore
             scr-set
             screen?
+            screen=?
             scrl
             scroll
             scrollok!
@@ -422,6 +423,7 @@
             cursyncup
             wenclose?
             window?
+            window=?
             syncdown
             syncup
 
