@@ -1,7 +1,7 @@
 /*
   menu_func.c
 
-  Copyright 2009, 2010, 2014, 2016 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2014, 2016, 2019 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -37,7 +37,6 @@
 #error "No menu.h file included"
 #endif
 
-#include "compat.h"
 #include "menu_func.h"
 #include "menu_type.h"
 #include "type.h"
@@ -45,13 +44,13 @@
 static void
 menu_posted_error (const char *funcname)
 {
-  scm_misc_error (funcname, "menu already posted", SCM_BOOL_F);
+  scm_misc_error (funcname, "menu already posted", SCM_EOL);
 }
 
 static void
 menu_not_connected_error (const char *funcname)
 {
-  scm_misc_error (funcname, "menu has no items", SCM_BOOL_F);
+  scm_misc_error (funcname, "menu has no items", SCM_EOL);
 }
 
 /* Set the foreground attribute of the menu */
@@ -218,7 +217,7 @@ gucu_menu_driver (SCM mnu, SCM c)
 {
   SCM_ASSERT (_scm_is_menu (mnu), mnu, SCM_ARG1, "menu-driver");
   SCM_ASSERT (scm_is_integer (c) || SCM_CHARP (c), c, SCM_ARG2,
-	      "menu-driver");
+              "menu-driver");
 
   int ret;
   SCM s_ret;
@@ -443,7 +442,7 @@ gucu_menu_request_name (SCM request)
   int c_request;
 
   SCM_ASSERT (scm_is_integer (request), request, SCM_ARG1,
-	      "menu-request-name");
+              "menu-request-name");
 
   c_request = scm_to_int (request);
 
@@ -493,7 +492,6 @@ gucu_set_menu_win (SCM menu, SCM win)
   MENU *c_menu;
   WINDOW *c_win;
   int ret;
-  struct gucu_menu *gm;
 
   SCM_ASSERT (_scm_is_menu (menu), menu, SCM_ARG1, "set-menu-win!");
   SCM_ASSERT (_scm_is_window (win), win, SCM_ARG2, "set-menu-win!");
@@ -506,8 +504,7 @@ gucu_set_menu_win (SCM menu, SCM win)
     menu_posted_error ("set-menu-win!");
 
   /* Protect the window from GC */
-  gm = (struct gucu_menu *) SCM_SMOB_DATA (menu);
-  gm->win_guard = win;
+  scm_foreign_object_set_x (menu, 1, SCM_UNPACK_POINTER (win));
 
   return SCM_UNSPECIFIED;
 }
@@ -519,7 +516,6 @@ gucu_set_menu_sub (SCM menu, SCM win)
   MENU *c_menu;
   WINDOW *c_win;
   int ret;
-  struct gucu_menu *gm;
 
   SCM_ASSERT (_scm_is_menu (menu), menu, SCM_ARG1, "set-menu-sub!");
   SCM_ASSERT (_scm_is_window (win), win, SCM_ARG2, "set-menu-sub!");
@@ -532,8 +528,7 @@ gucu_set_menu_sub (SCM menu, SCM win)
     menu_posted_error ("set-menu-sub!");
 
   /* Protect the window from GC */
-  gm = (struct gucu_menu *) SCM_SMOB_DATA (menu);
-  gm->subwin_guard = win;
+  scm_foreign_object_set_x (menu, 2, SCM_UNPACK_POINTER (win));
 
   return SCM_UNSPECIFIED;
 }
@@ -542,26 +537,14 @@ SCM
 gucu_menu_win (SCM arg1)
 {
   SCM_ASSERT (_scm_is_menu (arg1), arg1, SCM_ARG1, "menu-win");
-
-  struct gucu_menu *gm = (struct gucu_menu *) SCM_SMOB_DATA (arg1);
-
-  if (gm == NULL)
-    return SCM_BOOL_F;
-
-  return gm->win_guard;
+  return SCM_PACK_POINTER (scm_foreign_object_ref (arg1, 1));
 }
 
 SCM
 gucu_menu_sub (SCM arg1)
 {
   SCM_ASSERT (_scm_is_menu (arg1), arg1, SCM_ARG1, "menu-sub");
-
-  struct gucu_menu *gm = (struct gucu_menu *) SCM_SMOB_DATA (arg1);
-
-  if (gm == NULL)
-    return SCM_BOOL_F;
-
-  return gm->subwin_guard;
+  return SCM_PACK_POINTER (scm_foreign_object_ref (arg1, 2));
 }
 
 SCM
@@ -589,7 +572,8 @@ gucu_current_item (SCM arg1)
   ITEM *ret = current_item (c_arg1);
   if (ret != (ITEM *) 0)
     if (!item_increase_refcount (ret))
-      scm_misc_error ("gucu-current-item", "too many references to item", NULL);
+      scm_misc_error ("gucu-current-item", "too many references to item",
+                      SCM_EOL);
   SCM s_ret = _scm_from_item (ret);
 
   return s_ret;
@@ -788,7 +772,7 @@ gucu_menu_init_function ()
   scm_c_define_gsubr ("unpost-menu", 1, 0, 0, gucu_unpost_menu);
   scm_c_define_gsubr ("menu-request-name", 1, 0, 0, gucu_menu_request_name);
   scm_c_define_gsubr ("menu-request-by-name", 1, 0, 0,
-		      gucu_menu_request_by_name);
+                      gucu_menu_request_by_name);
   scm_c_define_gsubr ("set-menu-spacing!", 4, 0, 0, gucu_set_menu_spacing);
   scm_c_define_gsubr ("set-menu-win!", 2, 0, 0, gucu_set_menu_win);
   scm_c_define_gsubr ("set-menu-sub!", 2, 0, 0, gucu_set_menu_sub);
