@@ -1,7 +1,7 @@
 /*
   menu_type.c
 
-  Copyright 2009, 2010, 2011, 2014, 2016 Free Software Foundation, Inc.
+  Copyright 2009, 2010, 2011, 2014, 2016, 2019 Free Software Foundation, Inc.
 
   This file is part of GNU Guile-Ncurses.
 
@@ -38,24 +38,16 @@
 #error "No menu.h file included"
 #endif
 
-#include "compat.h"
 #include "menu_type.h"
 #include "type.h"
 
-scm_t_bits menu_tag;
-scm_t_bits item_tag;
+SCM menu_fo_type;
+SCM item_fo_type;
 
-SCM equalp_item (SCM x1, SCM x2);
-size_t gc_free_item (SCM x);
-SCM mark_item (SCM x);
-int print_item (SCM x, SCM port, scm_print_state * pstate);
+void gc_free_item (SCM x);
+void gc_free_menu (SCM x);
 
-SCM equalp_menu (SCM x1, SCM x2);
-size_t gc_free_menu (SCM x);
-SCM mark_menu (SCM x);
-int print_menu (SCM x, SCM port, scm_print_state * pstate);
-
-/* item -- in C, an ITEM *.  In Scheme, a smob that contains the
+/* item -- in C, an ITEM *.  In Scheme, a foreign object that contains the
  * pointer */
 
 void
@@ -133,9 +125,9 @@ gucu_new_item (SCM name, SCM description)
 int
 _scm_is_item (SCM x)
 {
-  if (SCM_SMOB_PREDICATE (item_tag, x))
+  if (SCM_IS_A_P (x, item_fo_type))
     {
-      if (SCM_SMOB_DATA (x) == 0)
+      if (scm_foreign_object_ref (x, 0) == NULL)
         return 0;
       else
         return 1;
@@ -147,52 +139,30 @@ _scm_is_item (SCM x)
 ITEM *
 _scm_to_item (SCM x)
 {
-  return (ITEM *) SCM_SMOB_DATA (x);
+  return (ITEM *) scm_foreign_object_ref (x, 0);
 }
 
 SCM
-_scm_from_item (ITEM * x)
+_scm_from_item (ITEM *x)
 {
   SCM s_item;
 
   assert (x != NULL);
 
-  SCM_NEWSMOB (s_item, item_tag, x);
+  s_item = scm_make_foreign_object_1 (item_fo_type, x);
 
-  assert (x == (ITEM *) SCM_SMOB_DATA (s_item));
+  assert (x == (ITEM *) scm_foreign_object_ref (s_item, 0));
 
   if (0)
     {
-      fprintf (stderr, "Making <#item> smob from ITEM * %p\n", (void *) x);
+      fprintf (stderr, "Making <#item> foreign object from ITEM * %p\n",
+               (void *) x);
     }
 
   return (s_item);
 }
 
-
-// Items are equal if they point to the same C structure
-SCM
-equalp_item (SCM x1, SCM x2)
-{
-  ITEM *item1 = (ITEM *) SCM_SMOB_DATA (x1);
-  ITEM *item2 = (ITEM *) SCM_SMOB_DATA (x2);
-
-  if ((item1 == NULL) || (item2 == NULL))
-    return SCM_BOOL_F;
-  else if ((item1 != item2))
-    return SCM_BOOL_F;
-  else
-    return SCM_BOOL_T;
-}
-
-SCM
-mark_item (SCM x UNUSED)
-{
-  // No SCMs in the item type: nothing to do here.
-  return (SCM_BOOL_F);
-}
-
-size_t
+void
 gc_free_item (SCM item)
 {
   ITEM *m = _scm_to_item (item);
@@ -203,51 +173,20 @@ gc_free_item (SCM item)
         {
           // Since no other #<menu> or #<item> is using the underlying
           // ITEM *, we can free it.
+
+          // Discarding this const qualifier below is intentional
           free (item_name (m));
           free (item_description (m));
           free_item (m);
-          SCM_SET_SMOB_DATA (item, 0);
+          scm_foreign_object_set_x (item, 0, NULL);
         }
       else
         {
           // Since some other #<menu> or #<item> has a reference to
           // this ITEM *, we just detach it.
-          SCM_SET_SMOB_DATA (item, 0);
+          scm_foreign_object_set_x (item, 0, NULL);
         }
     }
-
-  return 0;
-}
-
-int
-print_item (SCM x, SCM port, scm_print_state * pstate UNUSED)
-{
-  ITEM *item = (ITEM *) SCM_SMOB_DATA (x);
-  char str[SIZEOF_VOID_P*2+3];
-
-  assert (item != NULL);
-
-  scm_puts ("#<item ", port);
-
-  if (item == (ITEM *) NULL)
-    scm_puts ("(freed)", port);
-  else
-    {
-      if (item_name (item) != NULL)
-        {
-          scm_puts (item_name (item), port);
-          scm_puts (" ", port);
-        }
-
-      if (snprintf (str, sizeof(str), "%p", (void *) item) < 0)
-        scm_puts ("???", port);
-      else
-        scm_puts (str, port);
-    }
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
 }
 
 SCM
@@ -256,6 +195,12 @@ gucu_is_item_p (SCM x)
   return scm_from_bool (_scm_is_item (x));
 }
 
+SCM
+gucu_item_equal_p (SCM item1, SCM item2)
+{
+  return scm_from_bool (_scm_to_item (item1) == _scm_to_item (item2));
+}
+
 SCM
 gucu_item_refcount (SCM x)
 {
@@ -265,8 +210,9 @@ gucu_item_refcount (SCM x)
   return scm_from_int (item_get_refcount (m));
 }
 
-// menu -- in C, a MENU *.  In Scheme, a smob that contains the pointer
-// to a form along with a list that contains the SCM of the fields
+// menu -- in C, a MENU *.  In Scheme, a foreign object that contains
+// the pointer to a form along with a list that contains the SCM of
+// the fields
 
 // Note the C Menu item's internal list of fields must match the SCM
 // list of field to avoid garbage collection madness.
@@ -274,9 +220,9 @@ gucu_item_refcount (SCM x)
 int
 _scm_is_menu (SCM x)
 {
-  if (SCM_SMOB_PREDICATE (menu_tag, x))
+  if (SCM_IS_A_P (x, menu_fo_type))
     {
-      if (SCM_SMOB_DATA (x) == 0)
+      if (scm_foreign_object_ref (x, 0) == NULL)
         return 0;
       else
         return 1;
@@ -288,69 +234,38 @@ _scm_is_menu (SCM x)
 MENU *
 _scm_to_menu (SCM x)
 {
-  struct gucu_menu *gm;
+  scm_assert_foreign_object_type (menu_fo_type, x);
+  return (MENU *) scm_foreign_object_ref (x, 0);
+}
 
-  scm_assert_smob_type (menu_tag, x);
-
-  gm = (struct gucu_menu *) SCM_SMOB_DATA (x);
-  if (gm == NULL)
-    return NULL;
-
-  return gm->menu;
+SCM
+gucu_menu_equal_p (SCM menu1, SCM menu2)
+{
+  return scm_from_bool (_scm_to_menu (menu1) == _scm_to_menu (menu2));
 }
 
 // There is no _scm_from_menu since we need its items as a list to do
 // that
 // SCM _scm_from_menu (MENU *x)
 
-// Menus are equal if they point to the same C structure
-SCM
-equalp_menu (SCM x1, SCM x2)
-{
-  MENU *menu1 = _scm_to_menu (x1);
-  MENU *menu2 = _scm_to_menu (x2);
-
-  if ((menu1 == NULL) || (menu2 == NULL))
-    return SCM_BOOL_F;
-  else if ((menu1 != menu2))
-    return SCM_BOOL_F;
-  else
-    return SCM_BOOL_T;
-}
-
-SCM
-mark_menu (SCM x)
-{
-  struct gucu_menu *gm;
-
-  gm = (struct gucu_menu *) SCM_SMOB_DATA (x);
-  if (gm != NULL)
-    {
-      scm_gc_mark (gm->win_guard);
-      scm_gc_mark (gm->subwin_guard);
-    }
-  return SCM_BOOL_F;
-}
-
 /* The name is gc_free_menu because the curses primitive that frees
    memory is called free_menu. */
-size_t
+void
 gc_free_menu (SCM x)
 {
-  struct gucu_menu *gm;
   int retval;
   int i;
 
-  gm = (struct gucu_menu *) SCM_SMOB_DATA (x);
-  if (gm != NULL && gm->menu != NULL)
+  MENU *menu = (MENU *) scm_foreign_object_ref (x, 0);
+  if (menu != NULL)
     {
-      // First, we need make our own store the ITEM *.  If free_menu
+      // First, we need make our own store of the ITEM *.  If free_menu
       // succeeds, the list of ITEM * provided by menu_items will no
       // longer be valid.
-      int len = item_count (gm->menu);
-      ITEM **pitem = menu_items (gm->menu);
+      int len = item_count (menu);
+      ITEM **pitem = menu_items (menu);
       ITEM **pitem_store = scm_malloc (sizeof (ITEM *) * len);
-      for (i = 0; i < len; i ++)
+      for (i = 0; i < len; i++)
         pitem_store[i] = pitem[i];
 
       // Next, we try to free the menu.  Note that if the menu freeing
@@ -359,17 +274,19 @@ gc_free_menu (SCM x)
 
       int free_attempts = 0;
     freemenu:
-      retval = free_menu (gm->menu);
+      retval = free_menu (menu);
 
       if (retval == E_BAD_ARGUMENT)
         {
           free (pitem_store);
-          scm_misc_error ("garbage collection of menu", "bad argument", SCM_EOL);
+          scm_misc_error ("garbage collection of menu", "bad argument",
+                          SCM_EOL);
         }
       else if (retval == E_SYSTEM_ERROR)
         {
           free (pitem_store);
-          scm_misc_error ("garbage collection of menu", "system error", SCM_EOL);
+          scm_misc_error ("garbage collection of menu", "system error",
+                          SCM_EOL);
         }
       else if (retval == E_POSTED)
         {
@@ -379,14 +296,15 @@ gc_free_menu (SCM x)
           // mistake here.
           if (free_attempts == 0)
             {
-              unpost_menu (gm->menu);
-              free_attempts ++;
+              unpost_menu (menu);
+              free_attempts++;
               goto freemenu;
             }
           else
             {
               free (pitem_store);
-              scm_misc_error ("garbage collection of menu", "posted", SCM_EOL);
+              scm_misc_error ("garbage collection of menu", "posted",
+                              SCM_EOL);
             }
         }
 
@@ -398,7 +316,7 @@ gc_free_menu (SCM x)
       // is freed, its menu items still hold references to one another
       // to organize themselves in a doubly linked list, which frustrates
       // garbage collection.  This doubly-linked list deteched here.
-      for (i = 0; i < len; i ++)
+      for (i = 0; i < len; i++)
         {
           pitem_store[i]->left = NULL;
           pitem_store[i]->right = NULL;
@@ -407,7 +325,7 @@ gc_free_menu (SCM x)
         }
 
       // Decrease the refcount on these items, and maybe free them.
-      for (i = 0; i < len; i ++)
+      for (i = 0; i < len; i++)
         {
           if (!item_decrease_refcount (pitem_store[i]))
             {
@@ -426,43 +344,12 @@ gc_free_menu (SCM x)
       // Free our storage of the ITEM *
       free (pitem_store);
 
-      gm->menu = NULL;
+      scm_foreign_object_set_x (x, 0, NULL);
     }
 
-  /* Release the references holding the windows. */
-  if (gm != NULL)
-    {
-      gm->win_guard = SCM_BOOL_F;
-      gm->subwin_guard = SCM_BOOL_F;
-    }
-
-  SCM_SET_SMOB_DATA (x, NULL);
-
-  return 0;
-}
-
-int
-print_menu (SCM x, SCM port, scm_print_state * pstate UNUSED)
-{
-  MENU *menu = _scm_to_menu (x);
-  char str[SIZEOF_VOID_P*2+3];
-
-  scm_puts ("#<menu ", port);
-  if (menu == NULL)
-    {
-      scm_puts ("(freed) ", port);
-    }
-  else
-    {
-      if (snprintf (str, sizeof(str), "%p", (void *) menu) < 0)
-        scm_puts ("???", port);
-      else
-        scm_puts (str, port);
-    }
-  scm_puts (">", port);
-
-  // non-zero means success
-  return 1;
+  /* Release the references holding the window and subwindow. */
+  scm_foreign_object_set_x (x, 1, SCM_UNPACK_POINTER (SCM_BOOL_F));
+  scm_foreign_object_set_x (x, 2, SCM_UNPACK_POINTER (SCM_BOOL_F));
 }
 
 SCM
@@ -474,10 +361,9 @@ gucu_is_menu_p (SCM x)
 SCM
 gucu_new_menu (SCM items)
 {
-  struct gucu_menu *gm;
   size_t len;
   ITEM **c_items;
-  SCM smob;
+  SCM fobj;
   SCM entry;
   size_t i;
 
@@ -503,20 +389,21 @@ gucu_new_menu (SCM items)
       if (item_index (it) != ERR)
         scm_error_scm (scm_from_locale_symbol ("ncurses"),
                        scm_from_locale_string ("new-menu"),
-                       scm_from_locale_string ("~A is already assigned to a menu"),
-                       scm_list_1 (entry),
-                       SCM_BOOL_F);
+                       scm_from_locale_string
+                       ("~A is already assigned to a menu"),
+                       scm_list_1 (entry), SCM_BOOL_F);
     }
 
   // Step 1: allocate memory
-  gm = scm_gc_malloc (sizeof (struct gucu_menu), "gucu_menu");
-
-  c_items = scm_gc_malloc (sizeof (ITEM *) * (len + 1), "gucu_menu");
+  c_items = malloc (sizeof (ITEM *) * (len + 1));
 
   // Step 2: initialize it with C code
 
-  // Step 3: create the smob
-  SCM_NEWSMOB (smob, menu_tag, gm);
+  // Step 3: create the foreign object
+  fobj =
+    scm_make_foreign_object_3 (menu_fo_type, NULL,
+                               SCM_UNPACK_POINTER (SCM_BOOL_F),
+                               SCM_UNPACK_POINTER (SCM_BOOL_F));
 
   // Step 4: finish the initialization
   for (i = 0; i < len; i++)
@@ -536,14 +423,14 @@ gucu_new_menu (SCM items)
   /* This is a null-terminated array */
   c_items[len] = NULL;
 
-  gm->menu = new_menu (c_items);
+  MENU *menu = new_menu (c_items);
 
-  if (gm->menu == NULL)
+  if (menu == NULL)
     {
       free (c_items);
       if (errno == E_NOT_CONNECTED)
         {
-          scm_misc_error ("new-menu", "menu has no items", SCM_BOOL_F);
+          scm_misc_error ("new-menu", "menu has no items", SCM_EOL);
         }
       else if (errno == E_SYSTEM_ERROR)
         {
@@ -555,31 +442,27 @@ gucu_new_menu (SCM items)
       else
         abort ();
     }
-  scm_remember_upto_here_1 (items);
+  scm_foreign_object_set_x (fobj, 0, menu);
 
-  gm->win_guard = SCM_BOOL_F;
-  gm->subwin_guard = SCM_BOOL_F;
-
-  return smob;
+  return fobj;
 }
 
+#define makeFO(a,b,c) scm_make_foreign_object_type((a),(b),(c))
+#define u8sym(x) scm_from_utf8_symbol(x)
 void
 gucu_menu_init_type ()
 {
-  item_tag = scm_make_smob_type ("item", sizeof (ITEM *));
-  scm_set_smob_mark (item_tag, mark_item);
-  scm_set_smob_free (item_tag, gc_free_item);
-  scm_set_smob_print (item_tag, print_item);
-  scm_set_smob_equalp (item_tag, equalp_item);
+  item_fo_type = makeFO (u8sym ("item"),
+                         scm_list_1 (u8sym ("data")), gc_free_item);
   scm_c_define_gsubr ("item?", 1, 0, 0, gucu_is_item_p);
+  scm_c_define_gsubr ("item=?", 2, 0, 0, gucu_item_equal_p);
   scm_c_define_gsubr ("new-item", 2, 0, 0, gucu_new_item);
   scm_c_define_gsubr ("%item-refcount", 1, 0, 0, gucu_item_refcount);
 
-  menu_tag = scm_make_smob_type ("menu", sizeof (struct menu *));
-  scm_set_smob_mark (menu_tag, mark_menu);
-  scm_set_smob_free (menu_tag, gc_free_menu);
-  scm_set_smob_print (menu_tag, print_menu);
-  scm_set_smob_equalp (menu_tag, equalp_menu);
+  menu_fo_type = makeFO (u8sym ("menu"),
+                         scm_list_3 (u8sym ("menu"), u8sym ("window"),
+                                     u8sym ("subwindow")), gc_free_menu);
   scm_c_define_gsubr ("menu?", 1, 0, 0, gucu_is_menu_p);
+  scm_c_define_gsubr ("menu=?", 2, 0, 0, gucu_menu_equal_p);
   scm_c_define_gsubr ("new-menu", 1, 0, 0, gucu_new_menu);
 }
